import { jsxs as _jsxs, jsx as _jsx, Fragment as _Fragment } from "react/jsx-runtime";
import { SvgNmrSameTop, SvgNmrResetScale } from 'cheminfo-font';
import { memo, useCallback } from 'react';
import { FaCreativeCommonsSamplingPlus, FaEye, FaEyeSlash, } from 'react-icons/fa';
import { IoColorPaletteOutline } from 'react-icons/io5';
import { useChartData } from '../../context/ChartContext';
import { useDispatch } from '../../context/DispatchContext';
import Button from '../../elements/Button';
import { useAlert } from '../../elements/popup/Alert';
import { useModal } from '../../elements/popup/Modal';
import { useActiveSpectra } from '../../hooks/useActiveSpectra';
import useSpectrum from '../../hooks/useSpectrum';
import { getSpectraByNucleus } from '../../utility/getSpectraByNucleus';
import DefaultPanelHeader, { createFilterLabel, } from '../header/DefaultPanelHeader';
import { SpectraAutomaticPickingButton } from '../header/SpectraAutomaticPickingButton';
function getMissingProjection(SpectrumsData, activeTab) {
    let nucleus = activeTab.split(',');
    nucleus = nucleus[0] === nucleus[1] ? [nucleus[0]] : nucleus;
    const missingNucleus = [];
    for (const n of nucleus) {
        const hasSpectrums = SpectrumsData.some((d) => d.info.nucleus === n);
        if (!hasSpectrums) {
            missingNucleus.push(n);
        }
    }
    return missingNucleus;
}
function SpectraPanelHeaderInner({ data, activeSpectrum, activeSpectra, activeTab, displayerMode, onSettingClick, }) {
    const modal = useModal();
    const alert = useAlert();
    const dispatch = useDispatch();
    const handleDelete = useCallback(() => {
        modal.showConfirmDialog({
            message: (_jsxs("span", { children: ["You are about to delete", _jsxs("span", { style: { color: 'black' }, children: [" ", activeSpectra?.length, " "] }), "spectra, Are you sure?"] })),
            buttons: [
                {
                    text: 'Yes',
                    handler: () => {
                        dispatch({ type: 'DELETE_SPECTRA', payload: {} });
                    },
                },
                { text: 'No' },
            ],
        });
    }, [activeSpectra?.length, dispatch, modal]);
    function showAllSpectrumsHandler() {
        dispatch({
            type: 'CHANGE_SPECTRA_VISIBILITY_BY_NUCLEUS',
            payload: { nucleus: activeTab, flag: true },
        });
    }
    function hideAllSpectrumsHandler() {
        dispatch({
            type: 'CHANGE_SPECTRA_VISIBILITY_BY_NUCLEUS',
            payload: { nucleus: activeTab, flag: false },
        });
    }
    function addMissingProjectionHandler() {
        const missingNucleus = getMissingProjection(data, activeTab);
        if (missingNucleus.length > 0) {
            dispatch({
                type: 'ADD_MISSING_PROJECTION',
                payload: { nucleus: missingNucleus },
            });
        }
        else {
            alert.error('Nothing to calculate');
        }
    }
    function setSameTopHandler() {
        dispatch({ type: 'SET_SPECTRA_SAME_TOP' });
    }
    function resetScaleHandler() {
        dispatch({ type: 'RESET_SPECTRA_SCALE' });
    }
    function recolorSpectraHandler() {
        dispatch({
            type: 'RECOLOR_SPECTRA_COLOR',
            payload: {},
        });
    }
    const hasActiveSpectra = activeSpectra && activeSpectra?.length > 0;
    const spectraLengthPerTab = getSpectraByNucleus(activeTab, data)?.length;
    return (_jsxs(DefaultPanelHeader, { onDelete: handleDelete, counter: data?.length, counterLabel: createFilterLabel(data?.length, spectraLengthPerTab), deleteToolTip: "Delete selected spectra", disableDelete: !hasActiveSpectra, showSettingButton: true, onSettingClick: onSettingClick, children: [_jsx(Button.BarButton, { toolTip: "Hide selected spectra", onClick: hideAllSpectrumsHandler, tooltipOrientation: "horizontal", disabled: !hasActiveSpectra, children: _jsx(FaEyeSlash, {}) }), _jsx(Button.BarButton, { onClick: showAllSpectrumsHandler, toolTip: "Show selected spectra", tooltipOrientation: "horizontal", disabled: !hasActiveSpectra, children: _jsx(FaEye, {}) }), displayerMode === '2D' && activeSpectrum?.info.isFt && (_jsx(Button.BarButton, { toolTip: "Add missing projection", tooltipOrientation: "horizontal", onClick: addMissingProjectionHandler, children: _jsx(FaCreativeCommonsSamplingPlus, {}) })), displayerMode === '1D' && spectraLengthPerTab > 1 && (_jsxs(_Fragment, { children: [_jsx(Button.BarButton, { tooltipOrientation: "horizontal", toolTip: "Reset scale", onClick: resetScaleHandler, children: _jsx(SvgNmrResetScale, {}) }), _jsx(Button.BarButton, { tooltipOrientation: "horizontal", toolTip: "Same top", onClick: setSameTopHandler, children: _jsx(SvgNmrSameTop, {}) })] })), _jsx(SpectraAutomaticPickingButton, {}), _jsx(Button.BarButton, { tooltipOrientation: "horizontal", toolTip: "Recolor spectra", onClick: recolorSpectraHandler, children: _jsx(IoColorPaletteOutline, {}) })] }));
}
const MemoizedSpectraPanelHeader = memo(SpectraPanelHeaderInner);
export default function SpectrumsTabs({ onSettingClick, }) {
    const { data, view: { spectra: { activeTab }, }, displayerMode, } = useChartData();
    const spectrum = useSpectrum();
    const activeSpectra = useActiveSpectra();
    return (_jsx(MemoizedSpectraPanelHeader, { data,
        activeSpectrum: spectrum,
        activeSpectra,
        activeTab,
        displayerMode,
        onSettingClick }));
}
